@@ -38,7 +38,7 @@ pub trait ReadableRecord: Sized {
 
 /// Type definition of a generic record.
 /// A .dbf file is composed of many records
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone, Default)]
 pub struct Record {
     map: HashMap<String, FieldValue>,
 }
@@ -95,20 +95,30 @@ impl IntoIterator for Record {
     }
 }
 
-impl Default for Record {
-    fn default() -> Self {
-        Self {
-            map: Default::default(),
-        }
-    }
-}
-
 impl From<HashMap<String, FieldValue>> for Record {
     fn from(map: HashMap<String, FieldValue, RandomState>) -> Self {
         Self { map }
     }
 }
 
+impl From<Record> for HashMap<String, FieldValue> {
+    fn from(record: Record) -> HashMap<String, FieldValue> {
+        record.map
+    }
+}
+
+impl AsRef<HashMap<String, FieldValue>> for Record {
+    fn as_ref(&self) -> &HashMap<String, FieldValue> {
+        &self.map
+    }
+}
+
+impl AsMut<HashMap<String, FieldValue>> for Record {
+    fn as_mut(&mut self) -> &mut HashMap<String, FieldValue> {
+        &mut self.map
+    }
+}
+
 /// Structs containing the information allowing to
 /// create a new TableWriter which would write file
 /// with the same record structure as another dbase file.
@@ -210,10 +220,17 @@ impl<T: Read + Seek> Reader<T> {
 
     /// Creates an iterator of records of the type you want
     pub fn iter_records_as<R: ReadableRecord>(&mut self) -> RecordIterator<T, R> {
+        let record_size: usize = self
+            .fields_info
+            .iter()
+            .map(|i| i.field_length as usize)
+            .sum();
         RecordIterator {
             reader: self,
             record_type: std::marker::PhantomData,
             current_record: 0,
+            record_data_buffer: std::io::Cursor::new(vec![0u8; record_size]),
+            field_data_buffer: [0u8; 255],
         }
     }
 
@@ -247,11 +264,13 @@ impl<T: Read + Seek> Reader<T> {
         self.iter_records().collect::<Result<Vec<Record>, Error>>()
     }
 
-
     /// Seek to the start of the record at `index`
     pub fn seek(&mut self, index: usize) -> Result<(), Error> {
-        let offset = self.header.offset_to_first_record as usize + (index * self.header.size_of_record as usize);
-        self.source.seek(SeekFrom::Start(offset as u64)).map_err(|err| Error::io_error(err, 0))?;
+        let offset = self.header.offset_to_first_record as usize
+            + (index * self.header.size_of_record as usize);
+        self.source
+            .seek(SeekFrom::Start(offset as u64))
+            .map_err(|err| Error::io_error(err, 0))?;
         Ok(())
     }
 
@@ -276,7 +295,7 @@ impl<T: Read + Seek> Reader<T> {
     pub fn into_table_info(self) -> TableInfo {
         TableInfo {
             header: self.header,
-            fields_info: self.fields_info
+            fields_info: self.fields_info,
         }
     }
 }
@@ -342,11 +361,13 @@ pub struct NamedValue<'a, T> {
 /// will be returned.
 pub struct FieldIterator<'a, T: Read + Seek> {
     /// The source from where we read the data
-    pub(crate) source: &'a mut T,
+    pub(crate) source: &'a mut std::io::Cursor<Vec<u8>>,
     /// The fields that make the record
     pub(crate) fields_info: std::iter::Peekable<std::slice::Iter<'a, FieldInfo>>,
     /// The source where the Memo field data is read
     pub(crate) memo_reader: &'a mut Option<MemoReader<T>>,
+    /// Buffer where field data is stored
+    field_data_buffer: &'a mut [u8; 255],
 }
 
 impl<'a, T: Read + Seek> FieldIterator<'a, T> {
@@ -492,7 +513,9 @@ impl<'a, T: Read + Seek> FieldIterator<'a, T> {
 
     /// read the next field using the given info
     fn read_field(&mut self, field_info: &'a FieldInfo) -> Result<FieldValue, FieldIOError> {
-        match FieldValue::read_from(self.source, self.memo_reader, field_info) {
+        let field_data_buffer = &mut self.field_data_buffer[..field_info.length() as usize];
+        self.source.read_exact(field_data_buffer).unwrap();
+        match FieldValue::read_from(field_data_buffer, self.memo_reader, field_info) {
             Ok(value) => Ok(value),
             Err(kind) => Err(FieldIOError {
                 field: Some(field_info.clone()),
@@ -523,6 +546,10 @@ pub struct RecordIterator<'a, T: Read + Seek, R: ReadableRecord> {
     reader: &'a mut Reader<T>,
     record_type: std::marker::PhantomData<R>,
     current_record: u32,
+    record_data_buffer: std::io::Cursor<Vec<u8>>,
+    /// Non-Memo field length is stored on a u8,
+    /// so fields cannot exceed 255 bytes
+    field_data_buffer: [u8; 255],
 }
 
 impl<'a, T: Read + Seek, R: ReadableRecord> Iterator for RecordIterator<'a, T, R> {
@@ -532,10 +559,17 @@ impl<'a, T: Read + Seek, R: ReadableRecord> Iterator for RecordIterator<'a, T, R
         if self.current_record >= self.reader.header.num_records {
             None
         } else {
+            self.reader
+                .source
+                .read_exact(self.record_data_buffer.get_mut())
+                .ok()?;
+            self.record_data_buffer.set_position(0);
+
             let mut iter = FieldIterator {
-                source: &mut self.reader.source,
+                source: &mut self.record_data_buffer,
                 fields_info: self.reader.fields_info.iter().peekable(),
                 memo_reader: &mut self.reader.memo_reader,
+                field_data_buffer: &mut self.field_data_buffer,
             };
 
             let record = R::read_using(&mut iter)
